" Vim Keymap file for Vietnamese through VNI method
" Maintainer: Raphael McSinyx <vn.mcsinyx@gmail.com>
" Last Change: 2016-06-13

scriptencoding utf-8

let b:keymap_name = "vi"

loadkeymap

A\\	A
A1	Á
A2	À
A3	Ả
A4	Ã
A5	Ạ

A8	Ă
A8\\	Ă
A81	Ắ
A82	Ằ
A83	Ẳ
A84	Ẵ
A85	Ặ

A6	Â
A6\\	Â
A61	Ấ
A62	Ầ
A63	Ẩ
A64	Ẫ
A65	Ậ

D	D
D\\	D
D9	Đ

E\\	E
E1	É
E2	È
E3	Ẻ
E4	Ẽ
E5	Ẹ

E6	Ê
E6\\	Ê
E61	Ế
E62	Ề
E63	Ể
E64	Ễ
E65	Ệ

I\\	I
I1	Í
I2	Ì
I3	Ỉ
I4	Ĩ
I5	Ị

O\\	O
O1	Ó
O2	Ò
O3	Ỏ
O4	Õ
O5	Ọ

O6	Ô
O6\\	Ô
O61	Ố
O62	Ồ
O63	Ổ
O64	Ỗ
O65	Ộ

O7	Ơ
O7\\	Ơ
O71	Ớ
O72	Ờ
O73	Ở
O74	Ỡ
O75	Ợ

U\\	U
U1	Ú
U2	Ù
U3	Ủ
U4	Ũ
U5	Ụ

U7	Ư
U7\\	Ư
U71	Ứ
U72	Ừ
U73	Ử
U74	Ữ
U75	Ự

Y\\	Y
Y1	Ý
Y2	Ỳ
Y3	Ỷ
Y4	Ỹ
Y5	Ỵ

a\\	a
a1	á
a2	à
a3	ả
a4	ã
a5	ạ

a8	ă
a8\\	ă
a81	ắ
a82	ằ
a83	ẳ
a84	ẵ
a85	ặ

a6	â
a6\\	â
a61	ấ
a62	ầ
a63	ẩ
a64	ẫ
a65	ậ

d\\	d
d9	đ

e\\	e
e1	é
e2	è
e3	ẻ
e4	ẽ
e5	ẹ

e6	ê
e6\\	ê
e61	ế
e62	ề
e63	ể
e64	ễ
e65	ệ

i\\	i
i1	í
i2	ì
i3	ỉ
i4	ĩ
i5	ị

o\\	o
o1	ó
o2	ò
o3	ỏ
o4	õ
o5	ọ

o6	ô
o6\\	ô
o61	ố
o62	ồ
o63	ổ
o64	ỗ
o65	ộ

o7	ơ
o7\\	ơ
o71	ớ
o72	ờ
o73	ở
o74	ỡ
o75	ợ

u\\	u
u1	ú
u2	ù
u3	ủ
u4	ũ
u5	ụ

u7	ư
u7\\	ư
u71	ứ
u72	ừ
u73	ử
u74	ữ
u75	ự

y\\	y
y1	ý
y2	ỳ
y3	ỷ
y4	ỹ
y5	ỵ
